#ifndef __INIT_H__
#define __INIT_H__

#include "pic32_c.h"

/* Basic initialisation. */

void init_memory(void);
void init_pins(void);
void init_outputs(void);

/* Peripheral pin configuration. */

void config_uart(void);
void lock_config(void);
void unlock_config(void);

/* Convenience operations. */

void interrupts_on(void);

/* DMA configuration. */

void init_dma(void);

void dma_init(int channel, uint8_t pri);

void dma_set_auto_enable(int channel, int auto_enable);

void dma_set_chaining(int channel, enum dma_chain chain);

void dma_init_interrupt(int channel, uint8_t conditions,
                        uint8_t pri, uint8_t sub);

void dma_on(int channel);

void dma_set_interrupt(int channel, uint8_t int_num, int enable);

void dma_set_transfer(int channel,
                  uint32_t source_start_address, uint16_t source_size,
                  uint32_t destination_start_address, uint16_t destination_size,
                  uint16_t cell_size);

int DMA_INT_FLAGS(int channel, uint8_t flags);

uint32_t DMA_IPC_PRI(int channel, uint8_t pri, uint8_t sub);

/* Timer configuration. */

void timer_init(int timer, uint8_t prescale, uint16_t limit);

void timer_init_interrupt(int timer, uint8_t pri, uint8_t sub);

void timer_on(int timer);

int TIMER_INT_FLAGS(int timer, uint8_t flags);

uint32_t TIMER_IPC_PRI(int timer, uint8_t pri, uint8_t sub);
uint32_t TIMER_IPC_REG(int timer);

/* UART configuration. */

void uart_init(int uart, uint32_t baudrate);

void uart_init_interrupt(int uart, uint8_t conditions,
                         uint8_t pri, uint8_t sub);

void uart_on(int uart);

int UART_INT_FLAGS(int uart, uint8_t flags);

uint32_t UART_IPC_PRI(int uart, uint8_t pri, uint8_t sub);
uint32_t UART_IPC_REG(int uart);

#endif /* __INIT_H__ */
