#include "pic32_c.h"
#include "init.h"
#include "debug.h"

static const char message1[] = "Hello!\r\n";
static const char message2[] = "Again!\r\n";
static int uart_echo = 0;

static void blink(uint32_t delay, uint32_t port, uint32_t pins)
{
    uint32_t counter;

    /* Clear outputs (LED). */

    CLR_REG(port, pins);

    while (1)
    {
        counter = delay;

        while (counter--) __asm__("");	/* retain loop */

        /* Invert outputs (LED). */

        INV_REG(port, pins);
    }
}

void main(void)
{
    init_memory();
    init_pins();
    init_outputs();

    unlock_config();
    config_uart();
    lock_config();

    init_dma();

    /* Initiate DMA on the Timer2 interrupt. Since the channel is not
       auto-enabled, it must be explicitly enabled upon completion. */

    dma_init(0, 3);
    dma_set_interrupt(0, T2, 1);
    dma_set_transfer(0, PHYSICAL((uint32_t) message1), sizeof(message1) - 1,
                        HW_PHYSICAL(UART_REG(1, UxTXREG)), 1,
                        1);

    /* Enable DMA on the preceding channel's completion, with Timer3 initiating
       transfers, raising a transfer completion interrupt. */

    dma_init(1, 3);
    dma_set_chaining(1, dma_chain_previous);
    dma_set_interrupt(1, T3, 1);
    dma_set_transfer(1, PHYSICAL((uint32_t) message2), sizeof(message2) - 1,
                        HW_PHYSICAL(UART_REG(1, UxTXREG)), 1,
                        1);
    dma_init_interrupt(1, 0b00001000, 7, 3);

    /* Configure timers. */

    timer_init(2, 0b111, 60000);
    timer_init_interrupt(2, 1, 3);
    timer_on(2);

    timer_init(3, 0b111, 40000);
    timer_init_interrupt(3, 1, 3);
    timer_on(3);

    /* Set UART interrupt priority above CPU priority to process events. */

    uart_init(1, 115200);
    uart_init_interrupt(1, UxRIF, 7, 3);
    uart_on(1);

    interrupts_on();

    blink(3 << 24, PORTA, 1 << 3);
}

void exception_handler(void)
{
    blink(3 << 12, PORTA, 1 << 3);
}

void interrupt_handler(void)
{
    uint32_t ifs, val;

    /* Check for a UART receive interrupt condition (UxRIF). */

    ifs = REG(UARTIFS) & UART_INT_FLAGS(1, UxRIF);

    if (ifs)
    {
        /* Clear the UART interrupt condition. */

        CLR_REG(UARTIFS, ifs);

        /* Write the received data back. */

        while (REG(UART_REG(1, UxSTA)) & 1)
        {
            val = REG(UART_REG(1, UxRXREG));
            if (uart_echo)
                uart_write((char) val);

            /* Initiate transfer upon receiving a particular character. */

            if (val == '0')
                dma_on(0);
        }
    }

    /* Check for a DMA interrupt condition (CHBCIF). */

    ifs = REG(DMAIFS) & DMA_INT_FLAGS(1, 1);

    if (ifs)
    {
        INV_REG(PORTA, 1 << 2);
        CLR_REG(DMA_REG(0, DCHxINT), 0b11111111);
        CLR_REG(DMAIFS, ifs);

        dma_on(0);
    }
}
