#ifndef __ASSEMBLER__

#ifndef __PIC32_C_H__
#define __PIC32_C_H__

#include <stdint.h>
#include "mips.h"
#include "pic32.h"

/* Access. */

#define REG(mem)                *((volatile uint32_t *) (mem))

/* Bit clearing, setting and inverting. */

static inline void CLR_REG(uint32_t mem, uint32_t val)
{
    REG(mem + CLR) = val;
}

static inline void SET_REG(uint32_t mem, uint32_t val)
{
    REG(mem + SET) = val;
}

static inline void INV_REG(uint32_t mem, uint32_t val)
{
    REG(mem + INV) = val;
}

/* Address translation. */

static inline uint32_t PHYSICAL(uint32_t addr)
{
    return ((uint32_t) addr) - KSEG0_BASE;
}

static inline uint32_t HW_PHYSICAL(uint32_t addr)
{
    return ((uint32_t) addr) - KSEG1_BASE;
}

/* Register collection access. */

static inline uint32_t DMA_REG(int channel, uint32_t reg)
{
    return DCHBASE + reg + (channel - DCHMIN) * DCHSTEP;
}

static inline uint32_t TIMER_REG(int channel, uint32_t reg)
{
    return TIMERBASE + reg + (channel - TIMERMIN) * TIMERSTEP;
}

static inline uint32_t UART_REG(int channel, uint32_t reg)
{
    return UARTBASE + reg + (channel - UARTMIN) * UARTSTEP;
}

/* Convenience types. */

enum dma_chain
{
    dma_chain_none,
    dma_chain_next,
    dma_chain_previous,
};

#endif /* __PIC32_C_H__ */

#endif /* __ASSEMBLER__ */
